#!/usr/bin/env python

__author__    = 'Kurt Schwehr'
__version__   = '$Revision: 11970 $'.split()[1]
__revision__  = __version__ # For pylint
__date__ = '$Date: 2009-05-26 14:10:08 -0400 (Tue, 26 May 2009) $'.split()[1]
__copyright__ = '2008'
__license__   = 'GPL v3'
__contact__   = 'kurt at ccom.unh.edu'

__doc__='''

Create or add to a database.  First attempt.  This could be made better/faster.
Assumes that all ais messages have been "normalized" - messages are one line only.

Only deals with sqlite right now.

@requires: U{epydoc<http://epydoc.sourceforge.net/>} > 3.0
@requires: U{BitVector<http://cheeseshop.python.org/pypi/BitVector>}

@var __date__: Date of last svn commit
@undocumented: __version__ __author__ __doc__ parser
@status: Works, but not complete
'''

import sys
import traceback
import datetime
import exceptions

from decimal import Decimal
from BitVector import BitVector
import StringIO

import ais.binary    as binary
#import ais.aisstring as aisstring

import ais.ais_msg_1
import ais.ais_msg_2
import ais.ais_msg_3
import ais.ais_msg_4
import ais.ais_msg_5
import ais.ais_msg_18
#import ais.ais_msg_8
#import ais.ais_msg_21

import pysqlite2.dbapi2 as sqlite
import pysqlite2.dbapi2

def createTables(cx,verbose=False):
    '''
    param cx: database connection
    '''
    cu = cx.cursor()

    if verbose: print str(ais.ais_msg_1.sqlCreate(dbType='sqlite'))
    try:
        cu.execute(str(ais.ais_msg_1.sqlCreate(dbType='sqlite')))
    except:
        print 'Warning: position table already exists'

    # Skip 2 and 3 since they are also position messages
    
    if verbose: print str(ais.ais_msg_4.sqlCreate(dbType='sqlite'))
    cu.execute(str(ais.ais_msg_4.sqlCreate(dbType='sqlite')))

    if verbose: print str(ais.ais_msg_5.sqlCreate(dbType='sqlite'))
    cu.execute(str(ais.ais_msg_5.sqlCreate(dbType='sqlite')))

    cu.execute(str(ais.ais_msg_18.sqlCreate(dbType='sqlite')))

    cx.commit()

def loadData(cx,datafile,verbose=False
	     , uscg=True):
    '''
    Try to read data from an open file object.  Not yet well tested.

    @param cx: database connection
    @param verbose: pring out more if true
    @param uscg: Process uscg tail information to get timestamp and receive station
    @rtype: None
    @return: Nothing

    @note: can not handle multiline AIS messages.  They must be normalized first.
    '''
    cu = cx.cursor()
    lineNum = 0

    message_set = (1,2,3,4,5,18)
    counts = {}
    for msg_num in message_set:
        counts[msg_num] = 0

    for line in datafile:
	lineNum += 1
	if lineNum%1000==0:
	    print lineNum
	    cx.commit()

	fields=line.split(',') # FIX: use this split throughout below...

	if line[3:6] not in ('VDM|VDO'): continue # Not an AIS VHF message
	try:
	    msgNum = int(binary.ais6tobitvec(line.split(',')[5][0]))
	except:
	    print 'line would not decode',line
	    continue
	if verbose: print 'msgNum:',msgNum
	if msgNum not in message_set: 
	    if verbose: 
                print 'skipping',line
                print '  not in msg set:',str(message_set)
	    continue

	
	payload = bv = binary.ais6tobitvec(line.split(',')[5])

# FIX: need to take padding into account ... right before the *
	if msgNum in (1,2,3,4,18):
	    if len(bv) != 168:
		print 'ERROR: skipping bad one slot message, line:',lineNum
		print '  ',line,
		print '   Got length',len(bv), 'expected', 168
		continue
	elif msgNum == 5:
            # 426 has 2 pad bits
	    if len(bv) not in (424,426):
		print 'ERROR: skipping bad shipdata message, line:',lineNum
		print '  ',line,
		print '   Got length',len(bv), 'expected', 424
		continue
	    
        cg_sec = None
	cg_timestamp = None
	cg_station   = None

	if uscg:
            try:
                cg_sec = int(float(fields[-1])) # US Coast Guard time stamp.
                cg_timestamp = datetime.datetime.utcfromtimestamp(float(cg_sec))

            except Exception, e:
                print 'ERROR getting timestamp for',lineNum,line
                sys.stderr.write('    Exception:' + str(type(Exception))+'\n')
                sys.stderr.write('    Exception args:'+ str(e)+'\n')
                traceback.print_exc(file=sys.stderr)
                sys.exit('poof!')

	    for i in range(len(fields)-1,5,-1):
		if 0<len(fields[i]) and fields[i][0] in ('b','r','R'):
		    cg_station = fields[i]
		    break # Found it so ditch the for loop

	ins = None

	try:
	    if   msgNum==1: ins = ais.ais_msg_1.sqlInsert(ais.ais_msg_1.decode(bv),dbType='sqlite')
	    elif msgNum==2: ins = ais.ais_msg_2.sqlInsert(ais.ais_msg_2.decode(bv),dbType='sqlite')
	    elif msgNum==3: ins = ais.ais_msg_3.sqlInsert(ais.ais_msg_3.decode(bv),dbType='sqlite')
	    elif msgNum==4: ins = ais.ais_msg_4.sqlInsert(ais.ais_msg_4.decode(bv),dbType='sqlite')
	    elif msgNum==5: ins = ais.ais_msg_5.sqlInsert(ais.ais_msg_5.decode(bv),dbType='sqlite')
            elif msgNum==18:  ins = ais.ais_msg_18.sqlInsert(ais.ais_msg_18.decode(bv),dbType='sqlite') # Class B position
            else:
		print 'Warning... not handling type',msgNum,'line:',lineNum
		continue
	except:
	    print 'ERROR:  some decode error?','line:',lineNum
	    print '  ',line
	    continue

	counts[msgNum] += 1

	if uscg:
            # FIX: make cg_timestamp work
	    if None != cg_timestamp: ins.add('cg_timestamp',str(cg_timestamp))
	    if None != cg_station:   ins.add('cg_r',        cg_station)
	    if None != cg_sec:   ins.add('cg_sec',        cg_sec)
	if verbose: 
            print str(ins)
	try:
            cu.execute(str(ins))
        except pysqlite2.dbapi2.OperationalError, params:
	#except OperationalError,  params:
            if -1 != params.message.find('no such table'):
                print 'ERROR:',params.message
                sys.exit('You probably need to run with --with-create')
            print 'params',params
            print type(params)
	    print 'ERROR: sql error?','line:',lineNum
	    print '  ', str(ins)
	    print '  ',line

            if False:
                # Give some debugging flexibility
                from IPython.Shell import IPShellEmbed
                ipshell = IPShellEmbed(argv=[])
                ipshell() 
                sys.exit('Gave up')

    print counts
    cx.commit()
	

############################################################
if __name__=='__main__':
	from optparse import OptionParser
	parser = OptionParser(usage="%prog [options] file1.ais [file2.ais ...]",version="%prog "+__version__)

	parser.add_option('-d','--database-file',dest='databaseFilename',default='ais.db3',
			  help='Name of the python file to write [default: %default]')
	parser.add_option('-C','--with-create',dest='createTables',default=False, action='store_true',
			  help='Do not create the tables in the database')

	parser.add_option('--without-uscg',dest='uscgTail',default=True,action='store_false',
			  help='Do not look for timestamp and receive station at the end of each line [default: with-uscg]')

	parser.add_option('-v','--verbose',dest='verbose',default=False,action='store_true',
		help='Make program output more verbose info as it runs')

	(options,args) = parser.parse_args()
	cx = sqlite.connect(options.databaseFilename)

        #print 'FIX: cg_sec and cg_timestamp are currently broken'

	if options.createTables:
	    createTables(cx,verbose=options.verbose)

	if len(args)==0:
	    print 'processing from stdin'
    
	    loadData(cx,sys.stdin,verbose=options.verbose,uscg=options.uscgTail)
	else:
	    for filename in args:
		print 'processing file:',filename
		loadData(cx,file(filename,'r'),verbose=options.verbose,uscg=options.uscgTail)
